@@ -3,6 +3,9 @@
 //! shared between the various integration test runners, making sure that the integration tests run
 //! in exactly the same way as Monument itself.
 
+#![deny(clippy::all)]
+#![deny(rustdoc::broken_intra_doc_links)]
+
 pub mod calls;
 pub mod spec;
 
@@ -20,17 +23,18 @@ use bellframe::{
     place_not::{self, PnBlockParseError},
     InvalidRowError,
 };
-use log::{log, log_enabled, LogLevelFilter};
+use log::{log_enabled, LevelFilter};
 use monument::{Comp, Config, Progress, Query, QueryUpdate};
+use simple_logger::SimpleLogger;
 use spec::Spec;
 
-pub fn init_logging(log_level: LogLevelFilter) {
-    pretty_logger::init(
-        pretty_logger::Destination::Stderr,
-        log_level,
-        pretty_logger::Theme::default(),
-    )
-    .unwrap();
+pub fn init_logging(filter: LevelFilter) {
+    SimpleLogger::new()
+        .without_timestamps()
+        .with_colors(true)
+        .with_level(filter)
+        .init()
+        .unwrap();
 }
 
 pub fn run(
@@ -191,7 +195,7 @@ impl UpdateLogger {
 
     fn log(&mut self, update: QueryUpdate) -> Option<Comp> {
         // Early return if we can't log anything, making sure to still keep the composition
-        if !log_enabled!(log::LogLevel::Info) {
+        if !log_enabled!(log::Level::Info) {
             return match update {
                 QueryUpdate::Comp(c) => Some(c),
                 _ => None,
